@@ -2,26 +2,48 @@ rcbalance <-
 function(distance.structure, near.exact = NULL, fb.list = NULL, treated.info = NULL, control.info = NULL, exclude.treated = FALSE, target.group = NULL,  k = 1, penalty = 3){
 		
 ####################  CHECK INPUT #################### 
-	stopifnot(k > 0) 
-	stopifnot(penalty > 1)
+	if(!(k > 0)){
+		stop("k must be larger than zero")
+	}
+	if(!(penalty > 1)){
+		stop("penalty argument must be greater than 1")
+	}
 	#exclude.treated is incompatible with k > 1 and with target distributions other than default treated distribution
-	stopifnot(!(exclude.treated && (!is.null(target.group) || k > 1))) 
+	if(exclude.treated && (!is.null(target.group) || k > 1)){
+		stop("exclude.treated = TRUE is not compatible with target.group arguments or k arguments greater than 1")
+	}
 
 	if(!is.null(near.exact)){
-		stopifnot(!is.null(treated.info) && !is.null(control.info))
-		stopifnot(ncol(treated.info) == ncol(control.info))
-		stopifnot(all(colnames(treated.info) == colnames(control.info)))	
+		if(!(!is.null(treated.info) && !is.null(control.info))){
+			stop('treated.info and control.info must both be specified when near.exact is used')
+		}
+		if(ncol(treated.info) != ncol(control.info)){
+			stop('treated.info and control.info must have the same number of columns')
+		}
+		if(!all(colnames(treated.info) == colnames(control.info))){
+			stop('treated.info and control.info must have identical column names')
+		}	
 		treated.control.info <- rbind(treated.info, control.info)
 		if(class(distance.structure) %in% c('matrix', 'InfinitySparseMatrix', 'BlockedInfinitySparseMatrix')){		
-			stopifnot(nrow(treated.info) == nrow(distance.structure))
-			stopifnot(nrow(control.info) == ncol(distance.structure))
+			if(nrow(treated.info) != nrow(distance.structure)){
+				stop('Dimensions of treated.info and distance.structure do not agree')
+			}
+			if(nrow(control.info) != ncol(distance.structure)){
+				stop('Dimensions of control.info and distance.structure do not agree')				
+			}
 		}else{
 			#make sure number of treated in distance.structure and treated.info agree
-			stopifnot(nrow(treated.info) == length(distance.structure))
+			if(nrow(treated.info) != length(distance.structure)){
+				stop('treated.info and distance.structure specify different numbers of treated units')
+			}
 			#make sure number of controls in distance.structure and control.info agree, i.e. max control index in each list element must not exceed row count in matrix of all subjects
-			stopifnot(nrow(treated.control.info) >= max(laply(distance.structure, function(x) max(c(as.numeric(names(x)),0)))))		
+			if(nrow(control.info) < max(laply(distance.structure, function(x) max(c(as.numeric(names(x)),0))))){
+				stop('Not all control units in distance.structure have information in control.info')
+			}		
 		}	
-		stopifnot(all(near.exact %in% colnames(treated.info)))
+		if(!(all(near.exact %in% colnames(treated.info)))){
+			stop('near.exact contains variable names not present in colnames(treated.info)')
+		}
 	}
 	
 	if(!is.null(fb.list)){
@@ -33,22 +55,40 @@ function(distance.structure, near.exact = NULL, fb.list = NULL, treated.info = N
 		 	if(is.null(target.group)){
 		 		target.group <- treated.info
 		 	}
-		 	stopifnot(!is.null(target.group) && !is.null(control.info))
-			stopifnot(ncol(target.group) == ncol(control.info))
-			stopifnot(all(colnames(target.group) == colnames(control.info)))
+			if(!(!is.null(treated.info) && !is.null(control.info))){
+				stop('treated.info and control.info must both be specified when fb.list is used')
+			}
+			if(ncol(treated.info) != ncol(control.info)){
+				stop('treated.info and control.info must have the same number of columns')
+			}
+			if(!all(colnames(treated.info) == colnames(control.info))){
+				stop('treated.info and control.info must have identical column names')
+			}	
 			target.control.info <- rbind(target.group, control.info)
 			if(class(distance.structure) %in% c('matrix', 'InfinitySparseMatrix', 'BlockedInfinitySparseMatrix')){		
-				stopifnot(nrow(target.group) == nrow(distance.structure))
-				stopifnot(nrow(control.info) == ncol(distance.structure))
+				if(nrow(target.group) != nrow(distance.structure)){
+					stop('target.group and distance.structure dimensions do not agree')
+				}
+				if(nrow(control.info) != ncol(distance.structure)){
+					stop('control.info and distance.structure dimensions do not agree')					
+				}
 			}else{
 				#make sure number of treated in distance.structure and target.group agree
-				stopifnot(nrow(target.group) == length(distance.structure))
+				if(nrow(target.group) != length(distance.structure)){
+				stop('target.group and distance.structure specify different numbers of treated units')
+				}
 				#make sure number of controls in distance.structure and control.info agree, i.e. max control index in each list element must not exceed row count in matrix of all subjects
-				stopifnot(nrow(target.control.info) >= max(laply(distance.structure, function(x) max(c(as.numeric(names(x)),0)))))		
+				if(nrow(control.info) < max(laply(distance.structure, function(x) max(c(as.numeric(names(x)),0))))){
+					stop('Not all control units in distance.structure have information in control.info')
+				}		
 			}
-			stopifnot(all(unlist(fb.list) %in% colnames(target.group))) 
+			if(!all(unlist(fb.list) %in% colnames(target.group))){
+				stop('fb.list contains variable names not given in column names of other arguments')
+			} 
 			if(length(fb.list) > 1){
-				for(i in c(1:(length(fb.list)-1)))	stopifnot(all(fb.list[[i]] %in% fb.list[[i+1]]))	
+				for(i in c(1:(length(fb.list)-1)))	if(!all(fb.list[[i]] %in% fb.list[[i+1]])){
+					stop('Elements of fb.list must contain all variables listed in previous elements')
+				}	
 			}				
 		}	
 	}
@@ -83,8 +123,7 @@ function(distance.structure, near.exact = NULL, fb.list = NULL, treated.info = N
 
 ############################ RUN MATCH ##############################
 	if(any(is.na(as.integer(match.network$cost)))){
-		print('Integer overflow in penalty vector!  Run with a lower penalty value or fewer levels of fine balance.')
-		stop()
+		stop('Integer overflow in penalty vector!  Run with a lower penalty value or fewer levels of fine balance.')
 	}
 	o <- callrelax(match.network)	
 	if(o$feasible == 0){
@@ -106,7 +145,7 @@ function(distance.structure, near.exact = NULL, fb.list = NULL, treated.info = N
 	#make a |T| x k matrix with rownames equal to index of treated unit and indices of its matched controls stored in each row
 	x <- o$x[1:match.network$tcarcs]	
 	match.df <- data.frame('treat' = as.factor(match.network$startn[1:match.network$tcarcs]), 'x' = x, 'control' = match.network$endn[1:match.network$tcarcs])
-	matched.or.not <- daply(match.df, .(match.df$treat), function(treat.edges) c(treat.edges$treat[1], sum(treat.edges$x)))
+	matched.or.not <- daply(match.df, .(match.df$treat), function(treat.edges) c(as.numeric(as.character(treat.edges$treat[1])), sum(treat.edges$x)), .drop_o = FALSE)
 	if(any(matched.or.not[,2] == 0)){
 		match.df <- match.df[-which(match.df$treat %in% matched.or.not[which(matched.or.not[,2] == 0),1]),]
 	}
